import { NgModule, APP_INITIALIZER } from '@angular/core';

import axios from 'axios';

import { AXIOS_INSTANCE } from '@ucap/ng-core';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { environment } from '@environments';

import { AppAuthenticationGuard } from './guards/app-authentication.guard';

import { AppSessionResolver } from './resolvers/app-session.resolver';

import { AppAuthenticationService } from './services/app-authentication.service';
import { AppNotificationService } from './services/app-notification.service';
import { AppNativeService } from './services/app-native.service';
import { AppService } from './services/app.service';
import { AppChatService } from './services/app-chat.service';
import { AppFileService } from './services/app-file.service';
import { AppGroupService } from './services/app-group.service';
import { AppAccountService } from './services/app-account.service';

const GUARDS = [AppAuthenticationGuard];
const RESOLVERS = [AppSessionResolver];
const SERVICES = [
  AppService,
  AppAuthenticationService,
  AppNativeService,
  AppFileService,
  AppChatService,
  AppNotificationService,
  AppGroupService,
  AppAccountService
];

const axiosFactory = () => {
  const i = axios.create();

  i.defaults.timeout = 15 * 1000;

  return i;
};

const nativeServiceFactory = (nativeService: any) =>
  new environment.productConfig.nativeServiceClass(nativeService);

const appInit = (appService: AppService) => {
  return () => appService.initialize();
};

@NgModule({
  imports: [],
  exports: [],
  providers: [
    {
      provide: AXIOS_INSTANCE,
      useFactory: axiosFactory,
      deps: [],
      multi: false
    },
    {
      provide: UCAP_NATIVE_SERVICE,
      useFactory: nativeServiceFactory,
      deps: [AXIOS_INSTANCE],
      multi: false
    },
    {
      provide: APP_INITIALIZER,
      useFactory: appInit,
      deps: [AppService],
      multi: true
    },
    ...SERVICES,
    ...GUARDS,
    ...RESOLVERS
  ]
})
export class AppProviderModule {}
