import {
  Component,
  OnDestroy,
  OnInit,
  AfterViewInit,
  Renderer2
} from '@angular/core';

import { Store } from '@ngrx/store';

import { AppActions } from '@app/store/actions';
import { fromEvent, interval, Subject } from 'rxjs';
import { debounce, takeUntil } from 'rxjs/operators';
import { AppAuthenticationService } from './services/app-authentication.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent implements OnInit, OnDestroy, AfterViewInit {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private renderer2: Renderer2,
    private store: Store<any>,
    private appAuthenticationService: AppAuthenticationService
  ) {
    fromEvent(window, 'resize')
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        debounce(() => interval(100))
      )
      .subscribe((event: any) => {
        this.dispatchWindowSize({
          width: event.target.innerWidth,
          height: event.target.innerHeight
        });
      });

    // fromEvent(window, 'beforeunload')
    //   .pipe(takeUntil(this.ngOnDestroySubject))
    //   .subscribe((event: any) => {
    //     this.appAuthenticationService.logout();
    //   });
  }

  ngOnInit(): void {
    this.dispatchWindowSize({
      width: window.innerWidth,
      height: window.innerHeight
    });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit(): void {
    const preloader = this.renderer2.selectRootElement(
      '#ucap-lg-web-preloader'
    );
    this.renderer2.setStyle(preloader, 'display', 'none');
  }

  private dispatchWindowSize(size: { width: number; height: number }) {
    this.store.dispatch(AppActions.windowResized(size));
  }
}
