import { Component, ViewChild, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute, Params } from '@angular/router';

import { MatDrawer } from '@angular/material/sidenav';

import { Subscription, Subject, BehaviorSubject } from 'rxjs';
import { QueryParams } from '../types/params.type';
import { ChatDrawType } from '../types/chat-draw.type';
import { takeUntil } from 'rxjs/operators';
import { DrawInfo } from '../models/draw-info';
import { Store } from '@ngrx/store';
import { RoomActions, ChattingActions } from '@ucap/ng-store-chat';

@Component({
  selector: 'app-pages-chat-room',
  templateUrl: './chat-room.page.component.html',
  styleUrls: ['./chat-room.page.component.scss']
})
export class ChatRoomPageComponent implements OnInit, OnDestroy {
  isChatSearch = false;
  roomId: string;
  translationSimpleview = false;

  drawerType: ChatDrawType | null;
  returnDrawerType: ChatDrawType | null;
  eventSendTriggerSubject: BehaviorSubject<any> = new BehaviorSubject<any>(0);

  @ViewChild('chatRightDrawer', { static: false })
  chatRightDrawer: MatDrawer;

  ChatDrawType = ChatDrawType;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private activatedRoute: ActivatedRoute
  ) {}

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params: Params) => {
        const seqParam = params[QueryParams.ROOM_ID];
        // initializing by roomId Change.
        if (this.roomId !== seqParam) {
          if (!!this.chatRightDrawer) {
            this.chatRightDrawer.close();
          }
        }

        // setting roomId.
        this.roomId = !!seqParam ? seqParam : undefined;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.eventSendTriggerSubject) {
      this.eventSendTriggerSubject.complete();
    }

    // this.store.dispatch(RoomActions.clearSelectedRoom({ roomId: this.roomId }));
    this.store.dispatch(ChattingActions.clearActiveRoomId({}));
  }

  onRightDrawerToggle(type: DrawInfo | null): void {
    this.drawerType = type.chatDrawType;
    this.returnDrawerType = !!type.returnDrawType ? type.returnDrawType : null;
    this.chatRightDrawer.open();
  }
  onRightDrawerClose(): void {
    this.drawerType = null;
    this.returnDrawerType = null;
    this.chatRightDrawer.close();
  }
}
