import { of, Subject } from 'rxjs';
import { take, map, catchError } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  ViewChild,
  ChangeDetectionStrategy
} from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';

import { Store } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';

import { SearchData } from '@app/ucap/organization/models/search-data';

import { CreateDialogComponent } from '@app/sections/group/dialogs/create.dialog.component';
import { ListSectionComponent } from '@app/sections/group/components/list.section.component';
import { UserInfo } from '@ucap/protocol-sync';
import { SortViewType } from '../types/sort-view.type';

@Component({
  selector: 'app-pages-group-sidenav',
  templateUrl: './sidenav.page.component.html',
  styleUrls: ['./sidenav.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SidenavPageComponent implements OnInit, OnDestroy {
  @ViewChild('sectionGroupList', { static: false })
  sectionGroupList: ListSectionComponent;

  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;

  showType: SortViewType;
  sortViewType = SortViewType;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private activatedRoute: ActivatedRoute,
    private router: Router,
    private logService: LogService,
    private i18nService: I18nService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    public dialog: MatDialog
  ) {}

  ngOnInit(): void {
    this.showType = SortViewType.all;
    this.showGroupMenuIcon(SortViewType.all);
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickFab(event: MouseEvent) {}

  onClickGroupMenu(menuType: string) {
    switch (menuType) {
      case 'GROUP_NEW':
        {
          const dialogRef = this.dialog.open(CreateDialogComponent, {
            panelClass: 'max-create-dialog'
          });

          dialogRef
            .afterClosed()
            .pipe(
              take(1),
              map((result) => {}),
              catchError((err) => {
                return of(err);
              })
            )
            .subscribe();
        }
        break;
      case 'GROUP_EXPAND_MORE':
        {
          this.sectionGroupList.onExpandMore();
        }
        break;
      case 'GROUP_EXPAND_LESS':
        {
          this.sectionGroupList.onExpandLess();
        }
        break;
      case 'GROUP_CHANGE_ODER':
        {
        }
        break;
    }
  }

  onClickShowGroupMenu(menuType: SortViewType) {
    switch (menuType) {
      case SortViewType.all:
        {
          this.showType = SortViewType.all;
        }
        break;
      case SortViewType.onlineBuddy:
        {
          this.showType = SortViewType.onlineBuddy;
        }
        break;
      case SortViewType.onOff:
        {
          this.showType = SortViewType.onOff;
        }
        break;
    }
  }
  onSearchCancel() {
    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  showGroupMenuIcon(menuType: SortViewType): string {
    if (this.showType === menuType) {
      return 'check_circle';
    }

    return 'check_circle_outline';
  }

  onClickUser(userInfo: UserInfo) {
    this.router.navigate(
      [
        'group',
        {
          outlets: { content: 'index' }
        }
      ],
      {
        queryParams: { id: Number(userInfo.seq) }
      }
    );
  }
}
