import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output
} from '@angular/core';

import { LogService } from '@ucap/ng-logger';
import { RoomInfo } from '@ucap/protocol-room';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { QueryParams } from '@app/pages/chat/types/params.type';
import { SearchInfo } from '@app/pages/chat/models/search-info';

@Component({
  selector: 'app-sections-chat-list',
  templateUrl: './list.section.component.html',
  styleUrls: ['./list.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ListSectionComponent implements OnInit, OnDestroy {
  @Input()
  searchObj: SearchInfo;

  @Input()
  checkable = false;

  @Input()
  selectedRoomList: RoomInfo[] = [];

  @Output()
  searchResultList = new EventEmitter<RoomInfo[]>();

  @Output()
  toggleItem = new EventEmitter<{
    checked: boolean;
    roomInfo: RoomInfo;
  }>();

  currentRoomId: string;
  roomList: RoomInfo[];

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params: Params) => {
        const seqParam = params[QueryParams.ROOM_ID];
        this.currentRoomId = !!seqParam ? seqParam : undefined;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onToggleItem(event: { checked: boolean; roomInfo: RoomInfo }): void {
    this.toggleItem.emit(event);
  }

  onOpenChatRoom(roomInfo: RoomInfo): void {
    this.router.navigate(
      [
        'chat',
        {
          outlets: { content: 'chatroom' }
        }
      ],
      {
        queryParams: { roomId: roomInfo.roomId }
      }
    );
  }

  onSearchResultList(searchResultList: RoomInfo[]) {
    this.searchResultList.emit(searchResultList);
  }
}
