import { Subject } from 'rxjs';
import { tap, takeUntil } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { NativeService } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { LoginActions } from '@ucap/ng-store-authentication';
import { idleTimeChanged } from './actions';

@Injectable()
export class Effects {
  webLoginSuccessForNativeService$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(LoginActions.webLoginSuccess),
        tap((action) => {
          this.nativeService.app_postLogin();
        })
      ),
    { dispatch: false }
  );

  loginSuccessForNativeService$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(LoginActions.loginSuccess),
        tap((action) => {
          if (!this.logoutSubject) {
            this.logoutSubject = new Subject();
          }
          // this.nativeService.idle_startCheck(1000);

          // this.nativeService
          //   .idle_onState$()
          //   .pipe(takeUntil(this.logoutSubject))
          //   .subscribe((idle) => {
          //     console.log('idle', idle);
          //   });
        })
      ),
    { dispatch: false }
  );

  logoutSuccessForNativeService$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(LoginActions.logoutSuccess),
        tap((action) => {
          this.logoutSubject.next();
          this.logoutSubject.complete();
          this.logoutSubject = undefined;

          // this.nativeService.idle_stopCheck();

          this.nativeService.app_postLogout();
        })
      ),
    { dispatch: false }
  );

  idleTimeChanged$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(idleTimeChanged),
        tap((action) => {
          this.nativeService.idle_changeLimitTime(action.idleTime);
        })
      ),
    { dispatch: false }
  );

  private logoutSubject: Subject<void>;

  constructor(
    private actions$: Actions,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}
}
