import { map, tap } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';
import { Router } from '@angular/router';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { NativeService } from '@ucap/native';

import { LogService } from '@ucap/ng-logger';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { environment } from '@environments';
import { RoomActions } from '@ucap/ng-store-chat';

import { Store } from '@ngrx/store';

@Injectable()
export class Effects {
  createSuccess$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(RoomActions.createRoomSuccess),
        map((action) => action.res),
        tap((res) => {
          this.router.navigate(
            [
              'chat',
              {
                outlets: { content: 'chatroom' }
              }
            ],
            {
              queryParams: { roomId: res.roomId }
            }
          );
        })
      );
    },
    { dispatch: false }
  );

  selectedRoomSuccess$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(RoomActions.selectedRoomSuccess),
        tap((action) => {
          this.router.navigate(
            [
              'chat',
              {
                outlets: { content: 'chatroom' }
              }
            ],
            {
              queryParams: { roomId: action.roomId }
            }
          );
        })
      );
    },
    { dispatch: false }
  );

  clearSelectedRoom$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(RoomActions.clearSelectedRoom),
        tap(() => {
          this.router.navigate([
            'chat',
            {
              outlets: { content: 'index' }
            }
          ]);
        })
      );
    },
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private router: Router,
    private logService: LogService
  ) {}
}
