import { Subject, of, combineLatest } from 'rxjs';
import { takeUntil, take, map, catchError } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { FileInfo } from '@ucap/protocol-file';
import { LogService } from '@ucap/ng-logger';
import { FileDownloadItem } from '@ucap/api';
import { FileEventJson } from '@ucap/protocol-event';

@Component({
  selector: 'app-chat-attach-image-list',
  templateUrl: './attach-image-list.component.html',
  styleUrls: ['./attach-image-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AttachImageListComponent implements OnInit, OnDestroy {
  @Input()
  fileInfoList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];

  @Input()
  selectedList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];

  @Output()
  changeCheck = new EventEmitter<{
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }>();

  @Output()
  save = new EventEmitter<{
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }>();

  @Output()
  openViewer = new EventEmitter<FileInfo>();

  @Output()
  delete = new EventEmitter<FileInfo>();

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  getItemChecked(fileInfo: FileInfo): boolean {
    if (!!this.selectedList && this.selectedList.length > 0) {
      return this.selectedList.some(
        (info) => info.fileInfo.seq === fileInfo.seq
      );
    }
    return false;
  }

  getDownloadItem(fileInfo: FileInfo): FileDownloadItem {
    const obj = this.selectedList.find(
      (item) => item.fileInfo.seq === fileInfo.seq
    );

    if (!!obj) {
      return obj.fileDownloadItem;
    } else {
      return new FileDownloadItem();
    }
  }

  onChangeCheck(param: {
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }) {
    this.changeCheck.emit(param);
  }

  onClickSave(params: {
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }) {
    this.save.emit(params);
  }
  onClickOpenViewer(fileInfo: FileInfo) {
    this.openViewer.emit(fileInfo);
  }
  onClickDelete(fileInfo: FileInfo) {
    this.delete.emit(fileInfo);
  }
}
