import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { VersionInfo2Response } from '@ucap/api-public';
import { UserInfoSS } from '@ucap/protocol-query';

import { LogService } from '@ucap/ng-logger';
import { PresenceSelector } from '@ucap/ng-store-organization';
import { StatusBulkInfo } from '@ucap/protocol-status';

@Component({
  selector: 'app-group-profile-image-01',
  templateUrl: './profile-image-01.component.html',
  styleUrls: ['./profile-image-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileImage01Component implements OnInit, OnDestroy {
  @Input()
  set userInfo(userInfo: UserInfoSS) {
    this._userInfo = userInfo;
  }
  get userInfo(): UserInfoSS {
    return this._userInfo;
  }
  // tslint:disable-next-line: variable-name
  _userInfo: UserInfoSS;

  @Input()
  versionInfo: VersionInfo2Response;

  presenceInfo: StatusBulkInfo;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(
          PresenceSelector.selectStatusBulkInfo,
          Number(this.userInfo?.seq)
        )
      )
      .subscribe((status) => {
        this.presenceInfo = status as any;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }
}
