import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { VersionInfo2Response } from '@ucap/api-public';
import { UserInfoSS } from '@ucap/protocol-query';
import { StatusBulkInfo } from '@ucap/protocol-status';

import { LogService } from '@ucap/ng-logger';
import { PresenceSelector } from '@ucap/ng-store-organization';

@Component({
  selector: 'app-organization-profile-image-01',
  templateUrl: './profile-image-01.component.html',
  styleUrls: ['./profile-image-01.component.scss']
})
export class ProfileImage01Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoSS;

  @Input()
  versionInfo: VersionInfo2Response;

  @Output()
  openProfile: EventEmitter<UserInfoSS> = new EventEmitter();

  presenceInfo: StatusBulkInfo;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(
          PresenceSelector.selectStatusBulkInfo,
          Number(this.userInfo?.seq)
        )
      )
      .subscribe((status) => {
        this.presenceInfo = status;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoSS) {
    this.openProfile.emit(userInfo);
  }
}
