import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Output,
  EventEmitter,
  Input,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { Company } from '@ucap/api-external';

import { LogService } from '@ucap/ng-logger';
import { CompanySelector } from '@ucap/ng-store-organization';

import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { SearchData } from '../models/search-data';
import { UserStore } from '@app/models/user-store';
import { SearchForTenantComponent as UcapSearchForTenantComponent } from '@ucap/ng-ui-organization';

@Component({
  selector: 'app-organization-search-for-tenant',
  templateUrl: './search-for-tenant.component.html',
  styleUrls: ['./search-for-tenant.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SearchForTenantComponent implements OnInit, OnDestroy {
  @Output() searchDataChange: EventEmitter<SearchData> = new EventEmitter();
  @Input() set searchData(value: SearchData) {
    let data: SearchData = {
      ...value
    };

    if (!data.companyCode) {
      data = {
        ...data,
        companyCode: this.userStore.companyCode
      };
    }
    this._searchData = {
      ...data
    };
    if (!!this.searchForTenant && !!value && !!value.searchWord) {
      this.searchForTenant.changeSearchword(value.searchWord);
    }
  }
  get searchData() {
    return this._searchData;
  }
  // tslint:disable-next-line: variable-name
  _searchData: SearchData;

  @Output()
  canceled: EventEmitter<void> = new EventEmitter();

  @ViewChild('ucapOrgSearchForTenant', { static: true })
  searchForTenant: UcapSearchForTenantComponent;

  companyList: Company[];

  private ngOnDestroySubject: Subject<void> = new Subject();
  private userStore: UserStore;

  constructor(
    private appAuthenticationService: AppAuthenticationService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {
    this.userStore = this.appAuthenticationService.getUserStore();
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        this.companyList = companyList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChanged(data: { companyCode: string; searchWord: string }): void {
    if (!!data && data.searchWord.localeCompare('') === 0) {
      return;
    }
    this.searchDataChange.emit({
      companyCode: data.companyCode,
      searchWord: data.searchWord
    });
  }

  onCanceled(): void {
    this.searchForTenant.changeSearchword('');
    this.searchForTenant.inputSearchWord.nativeElement.value = '';

    this.canceled.emit();
  }
}
