import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  EventEmitter,
  Output,
  ViewChild,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { DeptInfo } from '@ucap/protocol-query';

import { LogService } from '@ucap/ng-logger';

import { DepartmentSelector } from '@ucap/ng-store-organization';

import { TreeComponent as UCAPTreeComponent } from '@ucap/ng-ui-organization';

import { environment } from '@environments';

@Component({
  selector: 'app-organization-tree',
  templateUrl: './tree.component.html',
  styleUrls: ['./tree.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class TreeComponent implements OnInit, OnDestroy {
  @Input()
  set initialExpanded(seq: number) {
    this.currentDeptSeq = seq;
    if (!!this.treeData) {
      this.expand(seq);
      return;
    }
    this._initialExpanded = seq;
  }
  // tslint:disable-next-line: variable-name
  _initialExpanded: number;

  @Output()
  clicked: EventEmitter<DeptInfo> = new EventEmitter();

  @ViewChild('treeList', { static: true })
  treeList: UCAPTreeComponent;

  treeData: {
    deptInfoList: DeptInfo[];
    displayRoot?: boolean;
    expanded?: number[];
  };

  currentDeptSeq: number;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    combineLatest([
      this.store.pipe(select(DepartmentSelector.departmentInfoList))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([deptInfoList]) => {
        this.treeData = {
          deptInfoList,
          displayRoot: environment.productConfig.organization.displayRoot,
          expanded: !!this._initialExpanded
            ? [this._initialExpanded]
            : undefined
        };
        this.changeDetectorRef.markForCheck();
        if (!!deptInfoList) {
          this._initialExpanded = undefined;
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickNode(node: DeptInfo) {
    this.currentDeptSeq = node.seq;
    this.clicked.emit(node);
  }

  expand(deptSeq: number) {
    this.treeList.expand(deptSeq);
  }

  expandAll() {
    this.treeList.expandAll();
  }

  collapse(deptSeq: number) {
    this.treeList.collapse(deptSeq);
  }

  collapseAll() {
    this.treeList.collapseAll();
  }
}
